
"""
Main entry: LO_server( PREFIX, PORT, Hidden=True )
"""

import sys

import subprocess
import os
# import signal

# import unohelper
# import string
# a UNO struct later needed to create a document
# from com.sun.star.beans import PropertyValue

# import pyuno
import time
# import socket  # only needed on win32-OOo3.0.0
import uno


# PORT='socket,host=localhost,port=2002'
# PORT='pipe,name=mypipename'

def _server_start( PREFIX, PORT, Hidden=True ):
    ARGS=[ os.path.join(PREFIX, 'soffice'),
           ("--accept=%s;urp;" % PORT),
           "--nologo",
           "--norestore"
           #, "--writer" ## should either create new or read existing explicitly
           #, "--nolockcheck"
               ]
    if Hidden:
        ARGS.append( "--headless" )
        ARGS.append( "--invisible" )
        pass
    p=subprocess.Popen( ARGS
                        , close_fds=True
                        #, stderr="/dev/null"
    )
    return p


def _server_stop(PREFIX, PORT, server_proc):
    subprocess.run( [ os.path.join(PREFIX, 'soffice')
                      ,("--unaccept=%s;urp;" % PORT) ] )
    if ( server_proc.poll() is None ):
        print( "sleep before kill" )
        time.sleep(0.1)
        server_proc.terminate()
        server_proc.wait(5)
        pass
    return

def _get_UnoUrlResolver( unoComponentContext ):
    u    = unoComponentContext # local context
    smgr = u.ServiceManager    # ServiceManager
    return smgr.createInstanceWithContext( "com.sun.star.bridge.UnoUrlResolver", u )

def _get_StarOffice_ComponentContext( unoUrlResolver, PORT ):
    return unoUrlResolver.resolve( "uno:%s;urp;StarOffice.ComponentContext" % PORT )

## Run F() repeatedly until it returns a non-None value.
## Return None if max_count attempts reached
## Otherwise return whatever F() returned
##
def try_until( label, FUN, max_count=100, sleep_time=0.1 ):
    count=0
    print("%s 000" % label, end='')
    res = None
    while (res is None) :
        try:
            res = FUN()
        except:
            time.sleep(sleep_time)
            count = count+1
            print( "\b\b\b%03d" % (count % 100) ,end='' )
            if ( count > max_count):
                print( " max_count reached, giving up" )
                return None
            res = None
    print( " done" )
    return res
#

class LO_server():
    """Connection to libreoffice.

    Fields:
         PREFIX
         PORT
         server_proc
         unoComponentContext
         unoUrlResolver
         ctx
         smgr
         desktop
    Methods:
         stop()
    """
    def __init__(self, PREFIX, PORT, Hidden=True ):
        if not (PREFIX in sys.path ): sys.path.append(PREFIX)
        self.PREFIX = PREFIX
        self.PORT   = PORT
        self.server_proc = _server_start( PREFIX, PORT, Hidden=Hidden )
        #
        # get the unoComponentContext (aka localContext) from the PyUNO runtime
        self.unoComponentContext = uno.getComponentContext()
        if ( self.unoComponentContext is None ):
            print("unoComponentContext = ", self.unoComponentContext)
        #
        # create the UnoUrlResolver
        self.unoUrlResolver = _get_UnoUrlResolver( self.unoComponentContext )
        if ( self.unoUrlResolver is None ):
            print( "unoUrlResolver = ", self.unoUrlResolver )
        #
        # connect to the running office
        #
        # print("get ctx")
        def F():
            return _get_StarOffice_ComponentContext( self.unoUrlResolver, self.PORT )
        #
        self.ctx = try_until( "ctx", F, max_count=100, sleep_time=0.1 )
        self.smgr = self.ctx.ServiceManager
        if ( self.smgr is None ): print("smgr = ", self.smgr)
        # get the central desktop object
        self.desktop = self.smgr.createInstanceWithContext( "com.sun.star.frame.Desktop",self.ctx)
        if ( self.desktop is None ): print("desktop = ", self.desktop)
        return
    #
    def stop(self):
        if ( self.server_proc is None ):
            print( "LO_server.close() : already closed" )
        else:
            _server_stop( self.PREFIX, self.PORT, self.server_proc )
            self.server_proc = None
            pass
        return
    #
    # Support for
    #   with LO_server( PREFIX, PORT, Hidden=True ) as lo:
    #        # work here
    #
    def __enter__(self):
        return self
    def __exit__(self, type, value, traceback):
        self.stop()
        return
#
